@@ -1,9 +1,10 @@
-/* $Id$ */
-
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
  *
+ *   Copyright (c) 2010
+ *   Canonical, Ltd. (All rights reserved)
+ *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of version 2 of the GNU General Public
  *   License published by the Free Software Foundation.
@@ -14,7 +15,8 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, contact Novell, Inc.
+ *   along with this program; if not, contact Novell, Inc. or Canonical,
+ *   Ltd.
  */
 
 #include <stdio.h>
@@ -26,6 +28,7 @@
 #include <fcntl.h>
 #include <mntent.h>
 #include <libintl.h>
+#include <linux/limits.h>
 #include <locale.h>
 #define _(s) gettext(s)
 
@@ -49,14 +52,13 @@
 #define MATCH_STRING "/sys/kernel/security/" MODULE_NAME "/matching"
 #define FLAGS_FILE "/sys/kernel/security/" MODULE_NAME "/features"
 #define MOUNTED_FS "/proc/mounts"
-#define PCRE "pattern=pcre"
 #define AADFA "pattern=aadfa"
 
 #define PRIVILEGED_OPS (kernel_load)
 #define UNPRIVILEGED_OPS (!(PRIVILEGED_OPS))
 
-const char *parser_title	= "Novell/SUSE AppArmor parser";
-const char *parser_copyright	= "Copyright (C) 1999, 2000, 2003, 2004, 2005, 2006 Novell Inc.";
+const char *parser_title	= "AppArmor parser";
+const char *parser_copyright	= "Copyright (C) 1999-2008 Novell Inc.\nCopyright 2009-2010 Canonical Ltd.";
 
 char *progname;
 int option = OPTION_ADD;
@@ -65,7 +67,7 @@ int binary_input = 0;
 int names_only = 0;
 int dump_vars = 0;
 int dump_expanded_vars = 0;
-dfaflags_t dfaflags = 0;
+dfaflags_t dfaflags = DFA_CONTROL_TREE_NORMAL | DFA_CONTROL_TREE_SIMPLE | DFA_CONTROL_MINIMIZE | DFA_CONTROL_MINIMIZE_HASH_TRANS | DFA_CONTROL_MINIMIZE_HASH_PERMS;
 int conf_verbose = 0;
 int conf_quiet = 0;
 int kernel_load = 1;
@@ -79,6 +81,7 @@ int read_implies_exec = 1;
 int read_implies_exec = 0;
 #endif
 int preprocess_only = 0;
+int skip_mode_force = 0;
 struct timespec mru_tstamp;
 
 char *subdomainbase = NULL;
@@ -87,8 +90,10 @@ char *flags_string = NULL;
 int regex_type = AARE_DFA;
 int perms_create = 0;		/* perms contain create flag */
 int kernel_supports_network = 1;	/* kernel supports network rules */
+int net_af_max_override = -1;		/* use kernel to determine af_max */
 char *profile_namespace = NULL;
 int flag_changehat_version = FLAG_CHANGEHAT_1_5;
+FILE *ofile = NULL;
 
 /* per-profile settings */
 int force_complain = 0;
@@ -109,6 +114,7 @@ struct option long_options[] = {
 	{"remove",		0, 0, 'R'},
 	{"names",		0, 0, 'N'},
 	{"stdout",		0, 0, 'S'},
+	{"ofile",		1, 0, 'o'},
 	{"match-string",	1, 0, 'm'},
 	{"quiet",		0, 0, 'q'},
 	{"skip-kernel-load",	0, 0, 'Q'},
@@ -149,6 +155,7 @@ static void display_usage(char *command)
 	       "-B, --binary		Input is precompiled profile\n"
 	       "-N, --names		Dump names of profiles in input.\n"
 	       "-S, --stdout		Dump compiled profile to stdout\n"
+	       "-o n, --ofile n		Write output to file n\n"
 	       "-b n, --base n		Set base dir and cwd\n"
 	       "-I n, --Include n	Add n to the search path\n"
 	       "-f n, --subdomainfs n	Set location of apparmor filesystem\n"
@@ -160,7 +167,7 @@ static void display_usage(char *command)
 	       "-T, --skip-read-cache	Do not attempt to load cached profiles\n"
 	       "-W, --write-cache	Save cached profile (force with -T)\n"
 	       "-q, --quiet		Don't emit warnings\n"
-	       "-v, --verbose		Show profile nams as they load\n"
+	       "-v, --verbose		Show profile names as they load\n"
 	       "-Q, --skip-kernel-load	Do everything except loading into kernel\n"
 	       "-V, --version		Display version info and exit\n"
 	       "-d, --debug 		Debug apparmor definitions\n"
@@ -171,30 +178,138 @@ static void display_usage(char *command)
 	       ,command);
 }
 
+/*
+ * flag: 1 - allow no- inversion
+ * flag: 2 - flags specified should be masked off
+ */
+typedef struct {
+	int control;
+	const char *option;
+	const char *desc;
+	dfaflags_t flags;
+} optflag_table_t;
+
+optflag_table_t dumpflag_table[] = {
+	{ 1, "rule-exprs", "Dump rule to expr tree conversions",
+	  DFA_DUMP_RULE_EXPR },
+	{ 1, "expr-stats", "Dump stats on expr tree", DFA_DUMP_TREE_STATS },
+	{ 1, "expr-tree", "Dump expression tree", DFA_DUMP_TREE },
+	{ 1, "expr-simplified", "Dump simplified expression tree",
+	  DFA_DUMP_SIMPLE_TREE },
+	{ 1, "stats", "Dump all compile stats",
+	  DFA_DUMP_TREE_STATS | DFA_DUMP_STATS | DFA_DUMP_TRANS_STATS |
+	  DFA_DUMP_EQUIV_STATS },
+	{ 1, "progress", "Dump progress for all compile phases",
+	  DFA_DUMP_PROGRESS | DFA_DUMP_STATS | DFA_DUMP_TRANS_PROGRESS |
+	  DFA_DUMP_TRANS_STATS },
+	{ 1, "dfa-progress", "Dump dfa creation as in progress",
+	  DFA_DUMP_PROGRESS | DFA_DUMP_STATS },
+	{ 1, "dfa-stats", "Dump dfa creation stats", DFA_DUMP_STATS },
+	{ 1, "dfa-states", "Dump dfa state diagram", DFA_DUMP_STATES },
+	{ 1, "dfa-graph", "Dump dfa dot (graphviz) graph", DFA_DUMP_GRAPH },
+	{ 1, "dfa-minimize", "Dump dfa minimization", DFA_DUMP_MINIMIZE },
+	{ 1, "dfa-unreachable", "Dump dfa unreachable states",
+	  DFA_DUMP_UNREACHABLE },
+	{ 1, "dfa-node-map", "Dump expr node set to state mapping",
+	  DFA_DUMP_NODE_TO_DFA },
+	{ 1, "dfa-uniq-perms", "Dump unique perms",
+	  DFA_DUMP_UNIQ_PERMS },
+	{ 1, "dfa-minimize-uniq-perms", "Dump unique perms post minimization",
+	  DFA_DUMP_MIN_UNIQ_PERMS },
+	{ 1, "compress-progress", "Dump progress of compression",
+	  DFA_DUMP_TRANS_PROGRESS | DFA_DUMP_TRANS_STATS },
+	{ 1, "compress-stats", "Dump stats on compression",
+	  DFA_DUMP_TRANS_STATS },
+	{ 1, "compressed-dfa", "Dump compressed dfa", DFA_DUMP_TRANS_TABLE },
+	{ 1, "equiv-stats", "Dump equivance class stats",
+	  DFA_DUMP_EQUIV_STATS },
+	{ 1, "equiv", "Dump equivance class", DFA_DUMP_EQUIV },
+	{ 0, NULL, NULL, 0 },
+};
+
+optflag_table_t optflag_table[] = {
+	{ 2, "0", "no optimizations",
+	  DFA_CONTROL_TREE_NORMAL | DFA_CONTROL_TREE_SIMPLE |
+	  DFA_CONTROL_MINIMIZE | DFA_CONTROL_REMOVE_UNREACHABLE
+	},
+	{ 1, "equiv", "use equivalent classes", DFA_CONTROL_EQUIV },
+	{ 1, "expr-normalize", "expression tree normalization",
+	  DFA_CONTROL_TREE_NORMAL },
+	{ 1, "expr-simplify", "expression tree simplification",
+	  DFA_CONTROL_TREE_SIMPLE },
+	{ 0, "expr-left-simplify", "left simplification first",
+	  DFA_CONTROL_TREE_LEFT },
+	{ 2, "expr-right-simplify", "right simplification first",
+	  DFA_CONTROL_TREE_LEFT },
+	{ 1, "minimize", "dfa state minimization", DFA_CONTROL_MINIMIZE },
+	{ 1, "hash-perms", "minimization - hash permissions during setup",
+	  DFA_CONTROL_MINIMIZE_HASH_PERMS },
+	{ 1, "hash-trans", "minimization - hash transitions during setup",
+	  DFA_CONTROL_MINIMIZE_HASH_TRANS },
+	{ 1, "remove-unreachable", "dfa unreachable state removal",
+	  DFA_CONTROL_REMOVE_UNREACHABLE },
+	{ 0, "compress-small",
+	  "do slower dfa transition table compression",
+	  DFA_CONTROL_TRANS_HIGH },
+	{ 2, "compress-fast", "do faster dfa transition table compression",
+	  DFA_CONTROL_TRANS_HIGH },
+	{ 0, NULL, NULL, 0 },
+};
+
+static void print_flag_table(optflag_table_t *table)
+{
+	int i;
+	unsigned int longest = 0;
+	for (i = 0; table[i].option; i++) {
+		if (strlen(table[i].option) > longest)
+			longest = strlen(table[i].option);
+	}
+
+	for (i = 0; table[i].option; i++) {
+		printf("%5s%-*s \t%s\n", (table[i].control & 1) ? "[no-]" : "",
+		       longest, table[i].option, table[i].desc);
+	}
+}
+
+static int handle_flag_table(optflag_table_t *table, const char *optarg,
+			     dfaflags_t *flags)
+{
+	const char *arg = optarg;
+	int i, invert = 0;
+
+	if (strncmp(optarg, "no-", 3) == 0) {
+		arg = optarg + 3;
+		invert = 1;
+	}
+
+	for (i = 0; table[i].option; i++) {
+		if (strcmp(table[i].option, arg) == 0) {
+			/* check if leading no- was specified but is not
+			 * supported by the option */
+			if (invert && !(table[i].control & 1))
+				return 0;
+			if (table[i].control & 2)
+				invert |= 1;
+			if (invert)
+				*flags &= ~table[i].flags;
+			else
+				*flags |= table[i].flags;
+			return 1;
+		}
+	}
+	return 0;
+}
+
 static void display_dump(char *command)
 {
 	display_version();
 	printf("\n%s: --dump [Option]\n\n"
 	       "Options:\n"
 	       "--------\n"
-	       "no option specified	Dump variables\n"
-	       "variables		Dump variables\n"
-	       "expanded-variables	Dump expanded variables\n"
-	       "expr-stats		Dump stats on expr tree\n"
-	       "expr-tree		Dump expression tree\n"
-	       "expr-simple		Dump simplified expression tree\n"
-	       "dfa-progress		Dump dfa creation as in progress\n"
-	       "dfa-stats		Dump dfa creation stats\n"
-	       "dfa-states		Dump dfa state diagram\n"
-	       "dfa-graph		Dump dfa dot (graphviz) graph\n"
-	       "dfa-minimize		Dump dfa minimization\n"
-	       "dfa-unreachable		Dump dfa unreachable states\n"
-	       "trans-progress		Dump progress of transition table\n"
-	       "trans-stats		Dump stats on transition table\n"
-	       "trans-table		Dump transition table\n"
-	       "equiv-stats		Dump equivance class stats\n"
-	       "equiv			Dump equivance class\n"
+	       "     none specified \tDump variables\n"
+	       "     variables      \tDump variables\n"
 	       ,command);
+	print_flag_table(dumpflag_table);
 }
 
 static void display_optimize(char *command)
@@ -203,21 +318,8 @@ static void display_optimize(char *command)
 	printf("\n%s: -O [Option]\n\n"
 	       "Options:\n"
 	       "--------\n"
-	       "0			no optimizations\n"
-	       "equiv			use equivalent classes\n"
-	       "no-equiv		don't use equivalent classes\n"
-	       "expr-normalize		do expr normalization\n"
-	       "expr-simplify		do expr tree simplification\n"
-	       "no-expr-normalize      	don't do expr normalization\n"
-	       "no-expr-simplify	don't do expr tree simplification\n"
-	       "expr-left-simplify	do left simplification first\n"
-	       "expr-right-simplify	do right simplification first\n"
-	       "no-minimize		don't do state minimization\n"
-	       "no-hash-part		don't hash partitions at start of minimization\n"
-	       "no-remove-unreachable	don't do unreachable state removal\n"
-	       "trans-comp-high		try to do extra transition table compression\n"
-	       "trans-comp-fast		do faster transition table compression\n"
 	       ,command);
+	print_flag_table(optflag_table);
 }
 
 void pwarn(char *fmt, ...)
@@ -251,7 +353,7 @@ static int process_args(int argc, char *argv[])
 	int count = 0;
 	option = OPTION_ADD;
 
-	while ((c = getopt_long(argc, argv, "adf:h::rRVvI:b:BCD:NSm:qQn:XKTWkO:p", long_options, &o)) != -1)
+	while ((c = getopt_long(argc, argv, "adf:h::rRVvI:b:BCD:NSm:qQn:XKTWkO:po:", long_options, &o)) != -1)
 	{
 		switch (c) {
 		case 0:
@@ -265,7 +367,7 @@ static int process_args(int argc, char *argv[])
 			break;
 		case 'd':
 			debug++;
-			skip_cache = 1;
+			skip_read_cache = 1;
 			break;
 		case 'h':
 			if (!optarg) {
@@ -292,6 +394,7 @@ static int process_args(int argc, char *argv[])
 		case 'R':
 			count++;
 			option = OPTION_REMOVE;
+			skip_cache = 1;
 			break;
 		case 'V':
 			display_version();
@@ -304,10 +407,12 @@ static int process_args(int argc, char *argv[])
 			set_base_dir(optarg);
 			break;
 		case 'B':
-			binary_input =1;
+			binary_input = 1;
+			skip_cache = 1;
 			break;
 		case 'C':
 			opt_force_complain = 1;
+			skip_cache = 1;
 			break;
 		case 'N':
 			names_only = 1;
@@ -319,92 +424,41 @@ static int process_args(int argc, char *argv[])
 			skip_read_cache = 1;
 			kernel_load = 0;
 			break;
+		case 'o':
+			count++;
+			option = OPTION_OFILE;
+			skip_read_cache = 1;
+			kernel_load = 0;
+			ofile = fopen(optarg, "w");
+			if (!ofile) {
+				PERROR("%s: Could not open file %s\n",
+				       progname, optarg);
+				exit(1);
+			}
+			break;
 		case 'f':
 			subdomainbase = strndup(optarg, PATH_MAX);
 			break;
 		case 'D':
-			skip_cache = 1;
+			skip_read_cache = 1;
 			if (!optarg) {
 				dump_vars = 1;
 			} else if (strcmp(optarg, "variables") == 0) {
 				dump_vars = 1;
 			} else if (strcmp(optarg, "expanded-variables") == 0) {
 				dump_expanded_vars = 1;
-			} else if (strcmp(optarg, "expr-tree") == 0) {
-				dfaflags |= DFA_DUMP_TREE;
-			} else if (strcmp(optarg, "expr-simple") == 0) {
-				dfaflags |= DFA_DUMP_SIMPLE_TREE;
-			} else if (strcmp(optarg, "expr-stats") == 0) {
-				dfaflags |= DFA_DUMP_TREE_STATS;
-			} else if (strcmp(optarg, "dfa-progress") == 0) {
-				dfaflags |= DFA_DUMP_PROGRESS | DFA_DUMP_STATS;
-			} else if (strcmp(optarg, "dfa-stats") == 0) {
-				dfaflags |= DFA_DUMP_STATS;
-			} else if (strcmp(optarg, "dfa-states") == 0) {
-				dfaflags |= DFA_DUMP_STATES;
-			} else if (strcmp(optarg, "dfa-graph") == 0) {
-				dfaflags |= DFA_DUMP_GRAPH;
-			} else if (strcmp(optarg, "dfa- minimize") == 0) {
-				dfaflags |= DFA_DUMP_MINIMIZE;
-			} else if (strcmp(optarg, "dfa-unreachable") == 0) {
-				dfaflags |= DFA_DUMP_UNREACHABLE;
-			} else if (strcmp(optarg, "trans-progress") == 0) {
-				dfaflags |= DFA_DUMP_TRANS_PROGRESS |
-				  DFA_DUMP_TRANS_STATS;
-			} else if (strcmp(optarg, "trans-stats") == 0) {
-				dfaflags |= DFA_DUMP_TRANS_STATS;
-			} else if (strcmp(optarg, "trans-table") == 0) {
-				dfaflags |= DFA_DUMP_TRANS_TABLE;
-			} else if (strcmp(optarg, "equiv") == 0) {
-				dfaflags |= DFA_DUMP_EQUIV;
-			} else if (strcmp(optarg, "equiv-stats") == 0) {
-				dfaflags |= DFA_DUMP_EQUIV_STATS;
-			} else {
+			} else if (!handle_flag_table(dumpflag_table, optarg,
+						      &dfaflags)) {
 				PERROR("%s: Invalid --Dump option %s\n",
 				       progname, optarg);
 				exit(1);
 			}
 			break;
 		case 'O':
-			skip_cache = 1;
-			if (strcmp(optarg, "0") == 0) {
-				dfaflags |= DFA_CONTROL_NO_TREE_NORMAL |
-					DFA_CONTROL_NO_TREE_SIMPLE |
-					DFA_CONTROL_NO_MINIMIZE |
-					DFA_CONTROL_NO_UNREACHABLE;
-			} else if (strcmp(optarg, "equiv") == 0) {
-				dfaflags |= DFA_CONTROL_EQUIV;
-			} else if (strcmp(optarg, "no-equiv") == 0) {
-				dfaflags &= ~DFA_CONTROL_EQUIV;
-			} else if (strcmp(optarg, "expr-normalize") == 0) {
-				dfaflags &= ~DFA_CONTROL_NO_TREE_NORMAL;
-			} else if (strcmp(optarg, "no-expr-normalize") == 0) {
-				dfaflags |= DFA_CONTROL_NO_TREE_NORMAL;
-			} else if (strcmp(optarg, "expr-simplify") == 0) {
-				dfaflags &= ~DFA_CONTROL_NO_TREE_SIMPLE;
-			} else if (strcmp(optarg, "no-expr-simplify") == 0) {
-				dfaflags |= DFA_CONTROL_NO_TREE_SIMPLE;
-			} else if (strcmp(optarg, "expr-left-simplify") == 0) {
-				dfaflags |= DFA_CONTROL_TREE_LEFT;
-			} else if (strcmp(optarg, "expr-right-simplify") == 0) {
-				dfaflags &= ~DFA_CONTROL_TREE_LEFT;
-			} else if (strcmp(optarg, "minimize") == 0) {
-				dfaflags &= ~DFA_CONTROL_NO_MINIMIZE;
-			} else if (strcmp(optarg, "no-minimize") == 0) {
-				dfaflags |= DFA_CONTROL_NO_MINIMIZE;
-			} else if (strcmp(optarg, "hash-part") == 0) {
-				dfaflags &= ~DFA_CONTROL_NO_HASH_PART;
-			} else if (strcmp(optarg, "no-hash-part") == 0) {
-				dfaflags |= DFA_CONTROL_NO_HASH_PART;
-			} else if (strcmp(optarg, "trans-comp-fast") == 0) {
-				dfaflags &= ~DFA_CONTROL_TRANS_HIGH;
-			} else if (strcmp(optarg, "trans-comp-high") == 0) {
-				dfaflags |= DFA_CONTROL_TRANS_HIGH;
-			} else if (strcmp(optarg, "remove-unreachable") == 0) {
-				dfaflags &= ~DFA_CONTROL_NO_UNREACHABLE;
-			} else if (strcmp(optarg, "no-remove-unreachable") == 0) {
-				dfaflags |= DFA_CONTROL_NO_UNREACHABLE;
-			} else {
+			skip_read_cache = 1;
+
+			if (!handle_flag_table(optflag_table, optarg,
+					       &dfaflags)) {
 				PERROR("%s: Invalid --Optimize option %s\n",
 				       progname, optarg);
 				exit(1);
@@ -447,6 +501,7 @@ static int process_args(int argc, char *argv[])
 			kernel_load = 0;
 			skip_cache = 1;
 			preprocess_only = 1;
+			skip_mode_force = 1;
 			break;
 		default:
 			display_usage(progname);
@@ -582,9 +637,6 @@ static void get_match_string(void) {
 
 out:
 	if (match_string) {
-		if (strstr(match_string, PCRE))
-			regex_type = AARE_PCRE;
-
 		if (strstr(match_string, AADFA))
 			regex_type = AARE_DFA;
 
@@ -640,20 +692,6 @@ fail:
 	return;
 }
 
-/* return 1 --> PCRE should work fine
-   return 0 --> no PCRE support */
-static int regex_support(void) {
-	/* no match string, predates (or postdates?) the split matching
-	module design */
-	if (!match_string)
-		return 1;
-
-	if (regex_type != AARE_NONE)
-		return 1;
-
-	return 0;
-}
-
 int process_binary(int option, char *profilename)
 {
 	char *buffer = NULL;
@@ -725,6 +763,27 @@ void reset_parser(char *filename)
 	reset_include_stack(filename);
 }
 
+int test_for_dir_mode(const char *basename, const char *linkdir)
+{
+	int rc = 0;
+
+	if (!skip_mode_force) {
+		char *target = NULL;
+		if (asprintf(&target, "%s/%s/%s", basedir, linkdir, basename) < 0) {
+			perror("asprintf");
+			exit(1);
+		}
+
+		if (access(target, R_OK) == 0)
+			rc = 1;
+
+		free(target);
+	}
+
+	return rc;
+}
+
+
 /* returns true if time is more recent than mru_tstamp */
 #define mru_t_cmp(a) \
 (((a).tv_sec == (mru_tstamp).tv_sec) ? \
@@ -746,6 +805,7 @@ int process_profile(int option, char *profilename)
 	char * cachename = NULL;
 	char * cachetemp = NULL;
 	char *basename = NULL;
+	FILE *cmd;
 
 	/* per-profile states */
 	force_complain = opt_force_complain;
@@ -758,42 +818,27 @@ int process_profile(int option, char *profilename)
 		}
 	}
 	else {
-		PERROR("%s: cannot use or update cache, disable, or force-complain via stdin\n", progname);
+		pwarn("%s: cannot use or update cache, disable, or force-complain via stdin\n", progname);
 	}
 
 	if (profilename && option != OPTION_REMOVE) {
 		/* make decisions about disabled or complain-mode profiles */
-		char *target = NULL;
 		basename = strrchr(profilename, '/');
 		if (basename)
 			basename++;
 		else
 			basename = profilename;
 
-		if (asprintf(&target, "%s/%s/%s", basedir, "disable", basename) < 0) {
-			perror("asprintf");
-			exit(1);
-		}
-
-		if (access(target, R_OK) == 0) {
-			if (!conf_quiet)
-				PERROR("Skipping profile in %s/disable: %s\n", basedir, basename);
-			free(target);
+		if (test_for_dir_mode(basename, "disable")) {
+ 			if (!conf_quiet)
+ 				PERROR("Skipping profile in %s/disable: %s\n", basedir, basename);
 			goto out;
 		}
-		free(target);
 
-		if (asprintf(&target, "%s/%s/%s", basedir, "force-complain", basename)<0) {
-			perror("asprintf");
-			exit(1);
-		}
-
-		if (access(target, R_OK) == 0) {
-			if (!conf_quiet)
-				PERROR("Warning: found %s in %s/force-complain, forcing complain mode\n", basename, basedir);
-			force_complain = 1;
-		}
-		free(target);
+		if (test_for_dir_mode(basename, "force-complain")) {
+			PERROR("Warning: found %s in %s/force-complain, forcing complain mode\n", basename, basedir);
+ 			force_complain = 1;
+ 		}
 
 		/* TODO: add primary cache check.
 		 * If .file for cached binary exists get the list of profile
@@ -808,6 +853,12 @@ int process_profile(int option, char *profilename)
 		update_mru_tstamp(yyin);
 	}
 
+	cmd = fopen("/proc/self/exe", "r");
+	if (cmd) {
+		update_mru_tstamp(cmd);
+		fclose(cmd);
+	}
+
 	retval = yyparse();
 	if (retval != 0)
 		goto out;
@@ -862,7 +913,7 @@ int process_profile(int option, char *profilename)
 		goto out;
 	}
 
-	retval = post_process_policy();
+	retval = post_process_policy(debug);
   	if (retval != 0) {
   		PERROR(_("%s: Errors found in file. Aborting.\n"), progname);
 		goto out;
@@ -879,10 +930,6 @@ int process_profile(int option, char *profilename)
 		goto out;
 	}
 
-	if (!regex_support()) {
-		die_if_any_regex();
-	}
-
 	retval = load_policy(option);
 
 out:
@@ -1028,5 +1075,8 @@ int main(int argc, char *argv[])
 		profilename = NULL;
 	}
 
+	if (ofile)
+		fclose(ofile);
+
 	return retval;
 }
