@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
@@ -19,6 +17,7 @@
 
 /* assistance routines */
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -27,16 +26,23 @@
 #define _(s) gettext(s)
 #include <netinet/in.h>
 #include <linux/socket.h>
+#include <linux/limits.h>
 #include <arpa/inet.h>
 #include <linux/capability.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 
 #include "parser.h"
 #include "parser_yacc.h"
 
 /* #define DEBUG */
 #ifdef DEBUG
+#undef PDEBUG
 #define PDEBUG(fmt, args...) printf("Lexer: " fmt, ## args)
 #else
+#undef PDEBUG
 #define PDEBUG(fmt, args...)	/* Do nothing */
 #endif
 #define NPDEBUG(fmt, args...)	/* Do nothing */
@@ -59,6 +65,7 @@ static struct keyword_table keyword_table[] = {
 	{"defined",		TOK_DEFINED},
 	{"change_profile",	TOK_CHANGE_PROFILE},
 	{"unsafe",		TOK_UNSAFE},
+	{"safe",		TOK_SAFE},
 	{"link",		TOK_LINK},
 	{"owner",		TOK_OWNER},
 	{"user",		TOK_OWNER},
@@ -102,7 +109,7 @@ static struct keyword_table rlimit_table[] = {
 };
 
 /* for alpha matches, check for keywords */
-static int get_table_token(const char *name, struct keyword_table *table,
+static int get_table_token(const char *name __unused, struct keyword_table *table,
 			   const char *keyword)
 {
 	int i;
@@ -142,42 +149,6 @@ int get_rlimit(const char *name)
 	return get_table_token("rlimit", rlimit_table, name);
 }
 
-static struct keyword_table address_family[] = {
-/*	{"unix",	AF_UNIX},
-	{"local",	AF_LOCAL},	*/
-	{"inet",	AF_INET},
-/*	{"ax25",	AF_AX25},
-	{"ipx",		AF_IPX},
-	{"appletalk",	AF_APPLETALK},
-	{"netrom",	AF_NETROM},
-	{"bridge",	AF_BRIDGE},
-	{"atmpvc",	AF_ATMPVC},
-	{"x25",		AF_X25}, */
-	{"inet6",	AF_INET6},
-/*	{"rose",	AF_ROSE},
-	{"decnet",	AF_DECnet},
-	{"netbeui",	AF_NETBEUI},
-	{"security",	AF_SECURITY},
-	{"key",		AF_KEY},
-	{"netlink",	AF_NETLINK},
-	{"route",	AF_ROUTE},
-	{"packet",	AF_PACKET},
-	{"ash",		AF_ASH},
-	{"econet",	AF_ECONET},
-	{"atmsvc",	AF_ATMSVC},
-	{"sna",		AF_SNA},
-	{"irda",	AF_IRDA},
-	{"pppox",	AF_PPPOX},
-	{"wanpipe",	AF_WANPIPE},
-	{"llc",		AF_LLC},
-	{"tipc",	AF_TIPC},
-	{"bluetooth",	AF_BLUETOOTH},
-	{"iucv",	AF_IUCV},
-	{"rxrpc",	AF_RXRPC}, */
-	/* terminate */
-	{NULL, 0}
-};
-
 struct network_tuple {
 	char *family_name;
 	unsigned int family;
@@ -236,6 +207,69 @@ static struct network_tuple network_mappings[] = {
 	{NULL, 0, NULL, 0, NULL, 0}
 };
 
+/* The apparmor kernel patches up until 2.6.38 didn't handle networking
+ * tables with sizes > AF_MAX correctly.  This could happen when the
+ * parser was built against newer kernel headers and then used to load
+ * policy on an older kernel.  This could happen during upgrades or
+ * in multi-kernel boot systems.
+ *
+ * Try to detect the running kernel version and use that to determine
+ * AF_MAX
+ */
+#define PROC_VERSION "/proc/sys/kernel/osrelease"
+static size_t kernel_af_max(void) {
+	char buffer[32];
+	int major;
+	int fd, res;
+
+	if (!net_af_max_override) {
+		return 0;
+	}
+	/* the override parameter is specifying the max value */
+	if (net_af_max_override > 0)
+		return net_af_max_override;
+
+	fd = open(PROC_VERSION, O_RDONLY);
+	if (!fd)
+		/* fall back to default provided during build */
+		return 0;
+	res = read(fd, &buffer, sizeof(buffer));
+	close(fd);
+	if (!res)
+		return 0;
+	buffer[sizeof(buffer)-1] = '\0';
+	res = sscanf(buffer, "2.6.%d", &major);
+	if (res != 1)
+		return 0;
+
+	switch(major) {
+	case 24:
+	case 25:
+	case 26:
+		return 34;
+	case 27:
+		return 35;
+	case 28:
+	case 29:
+	case 30:
+		return 36;
+	case 31:
+	case 32:
+	case 33:
+	case 34:
+	case 35:
+		return 37;
+	case 36:
+	case 37:
+		return 38;
+	/* kernels .38 and later should handle this correctly so no
+	 * static mapping needed
+	 */
+	default:
+		return 0;
+	}
+}
+
 /* Yuck. We grab AF_* values to define above from linux/socket.h because
  * they are more accurate than sys/socket.h for what the kernel actually
  * supports. However, we can't just include linux/socket.h directly,
@@ -246,13 +280,29 @@ static struct network_tuple network_mappings[] = {
  * hence the wrapping function.
  */
 size_t get_af_max() {
+	size_t af_max;
 	/* HACK: declare that version without "create" had a static AF_MAX */
-	if (!perms_create) return 36;
+	if (!perms_create && !net_af_max_override)
+		net_af_max_override = -1;
+
 #if AA_AF_MAX > AF_MAX
-	return AA_AF_MAX;
+	af_max = AA_AF_MAX;
 #else
-	return AF_MAX;
+	af_max = AF_MAX;
 #endif
+
+	/* HACK: some kernels didn't handle network tables from parsers
+	 * compiled against newer kernel headers as they are larger than
+	 * the running kernel expected.  If net_override is defined check
+	 * to see if there is a static max specified for that kernel
+	 */
+	if (net_af_max_override) {
+		size_t max = kernel_af_max();
+		if (max && max < af_max)
+			return max;
+	}
+
+	return af_max;
 }
 struct aa_network_entry *new_network_ent(unsigned int family,
 					 unsigned int type,
@@ -462,7 +512,7 @@ static void warn_uppercase(void)
 	}
 }
 
-static int parse_sub_mode(const char *str_mode, const char *mode_desc)
+static int parse_sub_mode(const char *str_mode, const char *mode_desc __unused)
 {
 
 #define IS_DIFF_QUAL(mode, q) (((mode) & AA_MAY_EXEC) && (((mode) & AA_EXEC_TYPE) != ((q) & AA_EXEC_TYPE)))
@@ -666,7 +716,6 @@ struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
 
 	entry->pattern_type = ePatternInvalid;
 	entry->pat.regex = NULL;
-	entry->pat.compiled = NULL;
 
 	entry->next = NULL;
 
@@ -692,7 +741,6 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 	/* XXX - need to create copies of the patterns, too */
 	entry->pattern_type = orig->pattern_type;
 	entry->pat.regex = NULL;
-	entry->pat.compiled = NULL;
 
 	entry->next = orig->next;
 
@@ -713,8 +761,6 @@ void free_cod_entries(struct cod_entry *list)
 		free(list->link_name);
 	if (list->pat.regex)
 		free(list->pat.regex);
-	if (list->pat.compiled)
-		free(list->pat.compiled);
 	free(list);
 }
 
