@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *   NOVELL (All rights reserved)
@@ -21,6 +19,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <libintl.h>
+#include <linux/limits.h>
 #define _(s) gettext(s)
 
 /* #define DEBUG */
@@ -34,38 +33,6 @@ enum error_type {
 	e_buffer_overflow
 };
 
-/* filter_escapes: scan input for any escape characters
- * and remove, and reduce double \\ to a single
- * NOTE: modifies in place the contents of the name argument */
-static void filter_escapes(char *name)
-{
-	char *sptr, *dptr;
-	BOOL bEscape = 0;
-
-	if (!name)	/* shouldn't happen */
-		return;
-
-	sptr = dptr = name;
-	while (*sptr) {
-		if (*sptr == '\\') {
-			if (bEscape) {
-				*dptr++ = *sptr++;
-			} else {
-				++sptr;
-				bEscape = TRUE;
-				continue;
-			}
-		} else if (dptr < sptr) {
-			*dptr++ = *sptr++;
-		} else {
-			dptr++;
-			sptr++;
-		}
-		bEscape = 0;
-	}
-	*dptr = 0;
-}
-
 /* Filters out multiple slashes (except if the first two are slashes,
  * that's a distinct namespace in linux) and trailing slashes.
  * NOTE: modifies in place the contents of the path argument */
@@ -74,7 +41,6 @@ static void filter_slashes(char *path)
 {
 	char *sptr, *dptr;
 	BOOL seen_slash = 0;
-	int len;
 
 	if (!path || (strlen(path) < 2))
 		return;
@@ -106,19 +72,6 @@ static void filter_slashes(char *path)
 		}
 	}
 	*dptr = 0;
-
-	if (regex_type != AARE_DFA) {
-		/* eliminate trailing slashes for versions of apparmor that
-		 * do not use the dfa engine.
-		 * Versions of apparmor which use the dfa engine use the
-		 * trailing / to differentiate between file and directory
-		 * matches
-		 */
-		len = strlen(path);
-		if (len > 2 && path[len -1] == '/') {
-			path[len - 1] = 0;
-		}
-	}
 }
 
 static pattern_t convert_aaregex_to_pcre(const char *aare, int anchor,
@@ -154,6 +107,9 @@ static pattern_t convert_aaregex_to_pcre(const char *aare, int anchor,
 	sptr = aare;
 	dptr = pcre;
 
+	if (dfaflags & DFA_DUMP_RULE_EXPR)
+		fprintf(stderr, "aare: %s   ->   ", aare);
+
 	if (anchor)
 		/* anchor beginning of regular expression */
 		*dptr++ = '^';
@@ -406,85 +362,13 @@ static pattern_t convert_aaregex_to_pcre(const char *aare, int anchor,
 out:
 	if (ret == FALSE)
 		ptype = ePatternInvalid;
+
+	if (dfaflags & DFA_DUMP_RULE_EXPR)
+		fprintf(stderr, "%s\n", pcre);
+
 	return ptype;
 }
 
-static int process_pcre_entry(struct cod_entry *entry)
-{
-	char tbuf[PATH_MAX + 3];	/* +3 for ^, $ and \0 */
-	int ret = TRUE;
-	pattern_t ptype;
-	int pos;
-	if (!entry) 		/* shouldn't happen */
-		return TRUE;
-
-	ptype = convert_aaregex_to_pcre(entry->name, 1, tbuf, PATH_MAX+3, &pos);
-	if (ptype == ePatternInvalid)
-		return FALSE;
-
-	entry->pattern_type = ptype;
-
-	/*
-	 * Only use buffer (tbuf) that we built above, if we
-	 * identified a pattern requiring full regex support.
-	 */
-	if (ptype == ePatternRegex) {
-		int pattlen = strlen(tbuf);
-
-		if ((entry->pat.regex = malloc(pattlen + 1))) {
-			const char *errorreason;
-			int errpos;
-
-			strcpy(entry->pat.regex, tbuf);
-
-			if ((entry->pat.compiled =
-			     pcre_compile(entry->pat.regex, 0,
-					  &errorreason, &errpos,
-					  NULL))) {
-				/* NULL out tables, kernel will use a
-				 * private version
-				 */
-				entry->pat.compiled->tables = NULL;
-			} else {
-				int i;
-
-				PERROR(_("%s: Failed to compile regex '%s' [original: '%s']\n"),
-				       progname, entry->pat.regex,
-				       entry->name);
-
-				PERROR(_("%s: error near               "),
-				       progname);
-
-				for (i = 0; i < errpos; i++) {
-					fputc('.', stderr);
-				}
-
-				fputc('^', stderr);
-				fputc('\n', stderr);
-
-				PERROR(_("%s: error reason: '%s'\n"),
-				       progname, errorreason);
-
-				free(entry->pat.regex);
-				entry->pat.regex = NULL;
-
-				ret = FALSE;
-			}
-		} else {
-			PERROR(_("%s: Failed to compile regex '%s' [original: '%s'] - malloc failed\n"),
-			       progname, entry->pat.regex, entry->name);
-
-			ret = FALSE;
-		}
-	} else {
-		/* not a regex, scan input for any escape characters
-		 * and remove, and reduce double \\ to a single */
-		filter_escapes(entry->name);
-	}		/* ptype == ePatternRegex */
-
-	return ret;
-}
-
 static const char *local_name(const char *name)
 {
 	const char *t;
@@ -502,14 +386,19 @@ static int process_profile_name_xmatch(struct codomain *cod)
 	const char *name;
 
 	/* don't filter_slashes for profile names */
-	name = local_name(cod->name);
+	if (cod->attachment)
+		name = cod->attachment;
+	else
+		name = local_name(cod->name);
 	ptype = convert_aaregex_to_pcre(name, 0, tbuf, PATH_MAX + 3,
 					&cod->xmatch_len);
+	if (ptype == ePatternBasic)
+		cod->xmatch_len = strlen(name);
 
 	if (ptype == ePatternInvalid) {
 		PERROR(_("%s: Invalid profile name '%s' - bad regular expression\n"), progname, name);
 		return FALSE;
-	} else if (ptype == ePatternBasic && !cod->altnames) {
+	} else if (ptype == ePatternBasic && !(cod->altnames || cod->attachment)) {
 		/* no regex so do not set xmatch */
 		cod->xmatch = NULL;
 		cod->xmatch_len = 0;
@@ -519,7 +408,7 @@ static int process_profile_name_xmatch(struct codomain *cod)
 		aare_ruleset_t *rule = aare_new_ruleset(0);
 		if (!rule)
 			return FALSE;
-		if (!aare_add_rule(rule, tbuf, 0, AA_MAY_EXEC, 0)) {
+		if (!aare_add_rule(rule, tbuf, 0, AA_MAY_EXEC, 0, dfaflags)) {
 			aare_delete_ruleset(rule);
 			return FALSE;
 		}
@@ -527,9 +416,15 @@ static int process_profile_name_xmatch(struct codomain *cod)
 			struct alt_name *alt;
 			list_for_each(cod->altnames, alt) {
 				int len;
-				convert_aaregex_to_pcre(alt->name, 0, tbuf,
-							PATH_MAX + 3, &len);
-				if (!aare_add_rule(rule, tbuf, 0, AA_MAY_EXEC, 0)) {
+				ptype = convert_aaregex_to_pcre(alt->name, 0,
+								tbuf,
+								PATH_MAX + 3,
+								&len);
+				if (ptype == ePatternBasic)
+					len = strlen(alt->name);
+				if (len < cod->xmatch_len)
+					cod->xmatch_len = len;
+				if (!aare_add_rule(rule, tbuf, 0, AA_MAY_EXEC, 0, dfaflags)) {
 					aare_delete_ruleset(rule);
 					return FALSE;
 				}
@@ -538,7 +433,7 @@ static int process_profile_name_xmatch(struct codomain *cod)
 		cod->xmatch = aare_create_dfa(rule, &cod->xmatch_size,
 					      dfaflags);
 		aare_delete_ruleset(rule);
- 		if (!cod->xmatch)
+		if (!cod->xmatch)
 			return FALSE;
 	}
 
@@ -581,11 +476,11 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 	if (entry->deny && (entry->mode & AA_LINK_BITS)) {
 		if (!aare_add_rule(dfarules, tbuf, entry->deny,
 				   entry->mode & ~AA_LINK_BITS,
-				   entry->audit & ~AA_LINK_BITS))
+				   entry->audit & ~AA_LINK_BITS, dfaflags))
 			return FALSE;
 	} else if (entry->mode & ~AA_CHANGE_PROFILE) {
 		if (!aare_add_rule(dfarules, tbuf, entry->deny, entry->mode,
-				   entry->audit))
+				   entry->audit, dfaflags))
 			return FALSE;
 	}
 
@@ -607,7 +502,7 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 			perms |= LINK_TO_LINK_SUBSET(perms);
 			vec[1] = "/[^/].*";
 		}
-		if (!aare_add_rule_vec(dfarules, entry->deny, perms, entry->audit & AA_LINK_BITS, 2, vec))
+		if (!aare_add_rule_vec(dfarules, entry->deny, perms, entry->audit & AA_LINK_BITS, 2, vec, dfaflags))
 			return FALSE;
 	}
 	if (entry->mode & AA_CHANGE_PROFILE) {
@@ -618,10 +513,10 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 			ptype = convert_aaregex_to_pcre(entry->namespace, 0, lbuf, PATH_MAX + 8, &pos);
 			vec[0] = lbuf;
 			vec[1] = tbuf;
-			if (!aare_add_rule_vec(dfarules, 0, AA_CHANGE_PROFILE, 0, 2, vec))
+			if (!aare_add_rule_vec(dfarules, 0, AA_CHANGE_PROFILE, 0, 2, vec, dfaflags))
 			    return FALSE;
 		} else {
-			if (!aare_add_rule(dfarules, tbuf, 0, AA_CHANGE_PROFILE, 0))
+		  if (!aare_add_rule(dfarules, tbuf, 0, AA_CHANGE_PROFILE, 0, dfaflags))
 				return FALSE;
 		}
 	}
@@ -631,10 +526,10 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 			char *vec[2];
 			vec[0] = entry->namespace;
 			vec[1] = entry->name;
-			if (!aare_add_rule_vec(dfarules, 0, mode, 0, 2, vec))
+			if (!aare_add_rule_vec(dfarules, 0, mode, 0, 2, vec, dfaflags))
 			    return FALSE;
 		} else {
-			if (!aare_add_rule(dfarules, entry->name, 0, mode, 0))
+		  if (!aare_add_rule(dfarules, entry->name, 0, mode, 0, dfaflags))
 				return FALSE;
 		}
 	}
@@ -643,18 +538,13 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 
 int post_process_entries(struct codomain *cod)
 {
-	int ret = TRUE, rc;
+	int ret = TRUE;
 	struct cod_entry *entry;
 	int count = 0;
 
 	list_for_each(cod->entries, entry) {
-		if (regex_type == AARE_DFA) {
-			rc = process_dfa_entry(cod->dfarules, entry);
-		} else {
-			filter_slashes(entry->name);
-			rc = process_pcre_entry(entry);
-		}
-		if (!rc)
+		if (regex_type == AARE_DFA &&
+		    !process_dfa_entry(cod->dfarules, entry))
 			ret = FALSE;
 		count++;
 	}
@@ -748,21 +638,6 @@ void yyerror(char *msg, ...)
 /* Guh, fake symbol */
 char *progname;
 
-static int test_filter_escapes(void)
-{
-	int rc = 0;
-	char *test_string;
-
-	test_string = strdup("foo\\\\foo");
-	filter_escapes(test_string);
-	MY_TEST(strcmp(test_string, "foo\\foo") == 0, "simple filter for \\\\");
-
-	test_string = strdup("foo\\foo");
-	filter_escapes(test_string);
-	MY_TEST(strcmp(test_string, "foofoo") == 0, "simple filter for \\f");
-	return rc;
-}
-
 static int test_filter_slashes(void)
 {
 	int rc = 0;
@@ -770,7 +645,7 @@ static int test_filter_slashes(void)
 
 	test_string = strdup("///foo//////f//oo////////////////");
 	filter_slashes(test_string);
-	MY_TEST(strcmp(test_string, "/foo/f/oo") == 0, "simple tests");
+	MY_TEST(strcmp(test_string, "/foo/f/oo/") == 0, "simple tests");
 
 	test_string = strdup("/foo/f/oo");
 	filter_slashes(test_string);
@@ -802,22 +677,16 @@ static int test_filter_slashes(void)
 
 	test_string = strdup("/a/");
 	filter_slashes(test_string);
-	MY_TEST(strcmp(test_string, "/a") == 0, "simple test for /a/");
+	MY_TEST(strcmp(test_string, "/a/") == 0, "simple test for /a/");
 
 	return rc;
 }
 
-int regex_type = AARE_PCRE;
-
 int main(void)
 {
 	int rc = 0;
 	int retval;
 
-	retval = test_filter_escapes();
-	if (retval != 0)
-		rc = retval;
-
 	retval = test_filter_slashes();
 	if (retval != 0)
 		rc = retval;
